﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Sample.ImageUploader.Entities
{
    internal class Report
    {
        [ApiMember(Description = "报告号")]
        public string ReportId { get; set; }
        
        #region 样品相关，和sample冗余

        [ApiMember(Description = "样品名称")]
        [StringLength(200), Required]
        public string SampleName { get; set; }

        /// <summary>
        /// 受理人姓名，不一定是Creator（从第三方系统进来的报告）
        /// </summary>
        [ApiMember(Description = "受理人")]
        [StringLength(20), Default("")]
        public string OperatorName { get; set; }

        //[ApiMember(Description = "样品Kvid")]
        [IgnoreUpdate]
        [Required]
        public Guid SampleKvid { get; set; }
        /// <summary>
        /// 样品名称
        /// </summary>

        /// <summary>
        /// 样品名称
        /// </summary>
        //[ApiMember(Description = "样品编号")]
        //[SerialNumber]
        [StringLength(50), Default("")]
        public string SampleSerialNumber { get; set; }

        /// <summary>
        /// 样品批次号
        /// </summary>
        //[ApiMember(Description = "样品批次号")]
        [StringLength(200), Default("")]
        public string SampleBatchNumber { get; set; }

        /// <summary>
        /// 样品品牌
        /// </summary>
        [ApiMember(Description = "品牌")]
        [StringLength(200), Default("")]
        public string SampleBrand { get; set; }

        /// <summary>
        /// 样品规格型号
        /// </summary>
        [ApiMember(Description = "规格型号")]
        [StringLength(200), Default("")]
        public string SampleModel { get; set; }

        /// <summary>
        /// 样品数量
        /// </summary>
        //[ApiMember(Description = "样品数量")]
        [DecimalLength(10, 2), Default(0)]
        public decimal SampleQuantity { get; set; }

        /// <summary>
        /// 数量单位，老数据的样品数量和单位记到metadata中了
        /// </summary>
        [ApiMember(Description = "数量单位")]
        [StringLength(50), Default("")]
        public string SampleQuantityUnit { get; set; }

        /// <summary>
        /// 样品品级
        /// </summary>
        [ApiMember(Description = "样品品级")]
        [StringLength(200), Default("")]
        public string SampleGrade { get; set; }

        /// <summary>
        /// 安全类别
        /// </summary>
        [ApiMember(Description = "安全类别")]
        [StringLength(200), Default("")]
        public string SampleGradeSafety { get; set; }

        /// <summary>
        /// 功能等级
        /// </summary>
        [ApiMember(Description = "功能等级")]
        [StringLength(200), Default("")]
        public string SampleGradeFunction { get; set; }

        public string SupplierName { get; set; }

        [ApiMember(Description = "委托单位")]
        public string DemanderName { get; set; }
        public string PayerName { get; set; }

        //[ApiMember(Description = "生产单位Kvid")]
        [DefaultEmptyGuid]
        public Guid SampleManufacturerKvid { get; set; }

        [ApiMember(Description = "生产单位")]
        [StringLength(100), Default("")]
        public string SampleManufacturerName { get; set; }

        [DefaultEmptyGuid]
        [IgnoreUpdate]
        public Guid SampleManufacturerContactKvid { get; set; }

        [StringLength(50), Default("")]
        public string SampleManufacturerContactName { get; set; }

        [StringLength(50), Default("")]
        //正则表达式验证：(^(\d{3,4}-)?\d{7,8})$|(13[0-9]{9})
        public string SampleManufacturerContactNumber { get; set; }

        [StringLength(200), Default("")]
        public string SampleManufacturerAddress { get; set; }
        #endregion

        #region 样品来源相关，和SampleSource冗余
        [DefaultEmptyGuid]
        public Guid SampleSourceKvid { get; set; }

        /// <summary>
        /// 样品来源编号，计划编号，快递号等
        /// </summary>
        [ApiMember(Description = "批号/货号")]
        [StringLength(200), Default("")]
        public string SampleSourceBatchNumber { get; set; }
        #endregion

        #region 报告受理，编制，审核，签发人员信息
        /// <summary>
        /// 受理人Kvid
        /// </summary>
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        /// <summary>
        /// 主检人
        /// </summary>
        [StringLength(50), Default("")]
        public string ManagerName { get; set; }

        /// <summary>
        /// 主检人Kvid
        /// </summary>
        [DefaultEmptyGuid]
        public Guid ManagerKvid { get; set; }

        /// <summary>
        /// 编制人Kvid
        /// </summary>
        [DefaultEmptyGuid]
        public Guid PreparerKvid { get; set; }

        /// <summary>
        /// 编制人姓名
        /// </summary>
        [StringLength(50), Default("")]
        public string PreparerName { get; set; }

        /// <summary>
        /// 审核人Kvid
        /// </summary>
        [DefaultEmptyGuid]
        public Guid ReviewerKvid { get; set; }

        /// <summary>
        /// 审核人姓名
        /// </summary>
        [StringLength(50), Default("")]
        public string ReviewerName { get; set; }

        /// <summary>
        /// 签发人Kvid
        /// </summary>
        [DefaultEmptyGuid]
        public Guid IssuerKvid { get; set; }

        /// <summary>
        /// 签发人姓名
        /// </summary>
        [StringLength(50), Default("")]
        public string IssuerName { get; set; }

        public int Status { get; set; }
        #endregion
    }
}
